@@ -1,10 +1,11 @@
-/* --< GKrellAclock 0.3.2 >--{ 13 August 2002 }--
+/* --< GKrellAclock 0.3.3 >--{ 22 Jan 2005 }--
  * Ported to GkrellM2.0
  * includes Bill Nalen's windows port.
  *
  * Author: M.R.Muthu Kumar (m_muthukumar@users.sourceforge.net)
  *
  * All the graphic routines are from - Tom Gilbert ( http://linuxbrit.co.uk/ )
+ * Xclock code contributed by Stephan Bourgeois <strangelv@lycos.co.uk>
  */
 
 #if defined(WIN32)
@@ -26,7 +27,7 @@
 
 #include<math.h>
 
-#define GKRELLACLOCK_VER 	"0.3.2"
+#define GKRELLACLOCK_VER 	"0.3.3"
 
 #define	CONFIG_NAME	"GkrellAclock"	/* Name in the configuration window */
 #define	STYLE_NAME	"GkrellAclock"	/* Theme subdirectory name */
@@ -55,20 +56,24 @@
 #define N_GRAY   "Gray"
 #define N_L_PINK "Light Pink"
 
+#define R_CLOCK 0
+#define X_CLOCK 1
+
 static GkrellmMonitor		*mon;
 static GkrellmChart		*chart;
 static GkrellmChartconfig	*chart_config=NULL;
 
-static GtkWidget    *cycle_option,
-					*sec_select_option,
-				    *dial_select_option;
+static GtkWidget	*cycle_option,
+			*sec_select_option,
+			*dial_select_option,
+			*clock_type_option[2];
 
 static gchar	*color_name[] = { N_WHITE, N_ORANGE, N_GREEN, N_YELLOW, 
                                   N_CYAN, N_RED, N_GRAY, N_L_PINK };
 
 static gint c_red[MAX_COLORS], c_green[MAX_COLORS], c_blue[MAX_COLORS];
 
-static gint d_color, s_color, cycle;
+static gint d_color, s_color, cycle, clock_type;
 static gint prev_h = -1;
 static gint chart_w = 60;
 
@@ -308,6 +313,197 @@ aa_line (gint x1, gint y1, gint x2, gint y2, guchar b,
     }
 }
 
+
+static void
+Wu_line (gint x1, gint y1, gint x2, gint y2, guchar b,
+         guchar rr, guchar gg, guchar bb)
+{
+  gdouble grad, xd, yd, xgap, ygap, xend, yend, yf, xf,
+    brightness1, brightness2, db;
+  gint ix1, ix2, iy1, iy2, i;
+  gint temp;
+
+  guchar c1, c2;
+
+  xd = (x2 - x1);
+  yd = (y2 - y1);
+
+  if (abs (xd) > abs (yd))
+    {
+      if (x1 > x2)
+        {
+          temp = x1;
+          x1 = x2;
+          x2 = temp;
+          temp = y1;
+          y1 = y2;
+          y2 = temp;
+          xd = (x2 - x1);
+          yd = (y2 - y1);
+        }
+
+      grad = yd/xd;
+
+/* end point 1 */
+
+      xend = (int) x1 + 0.5;
+      yend = y1 + grad * (xend - x1);
+
+      xgap = (1 - modf (x1 + 0.5, &db));
+      ix1 = (int) xend;
+      iy1 = (int) yend;
+
+      brightness1 = (1 - modf (yend, &db)) * xgap;
+      brightness2 = modf (yend, &db) * xgap;
+
+      c1 = (unsigned char) (brightness1 * b);
+      c2 = (unsigned char) (brightness2 * b);
+
+      set_col_pixel (ix1, iy1, c1, rr, gg, bb);
+      set_col_pixel (ix1, iy1 + 1, c2, rr, gg, bb);
+
+      yf = yend + grad;
+      
+/* end point 2 */      
+
+      xend = (int) (x2 + .5);
+      yend = y2 + grad * (xend - x2);
+
+      xgap = 1 - modf (x2 - .5, &db);
+
+      ix2 = (int) xend;
+      iy2 = (int) yend;
+
+      brightness1 = (1 - modf (yend, &db)) * xgap;
+      brightness2 = modf (yend, &db) * xgap;
+
+      c1 = (unsigned char) (brightness1 * b);
+      c2 = (unsigned char) (brightness2 * b);
+
+      set_col_pixel (ix2, iy2, c1, rr, gg, bb);
+      set_col_pixel (ix2, iy2 + 1, c2, rr, gg, bb);
+      
+/* main loop */
+
+      for (i = ix1 + 1; i < ix2; i++)
+        {
+          brightness1 = (1 - modf (yf, &db));
+          brightness2 = modf (yf, &db);
+          c1 = (unsigned char) (brightness1 * b);
+          c2 = (unsigned char) (brightness2 * b);
+
+          set_col_pixel (i, (int) yf, c1, rr, gg, bb);
+          set_col_pixel (i, (int) yf + 1, c2, rr, gg, bb);
+
+          yf = yf + grad;
+        }
+    }
+  else
+    {
+      if (y1 > y2)
+        {
+          temp = x1;
+          x1 = x2;
+          x2 = temp;
+          temp = y1;
+          y1 = y2;
+          y2 = temp;
+          xd = (x2 - x1);
+          yd = (y2 - y1);
+        }
+
+      grad = xd / yd;
+
+      yend = (int) (y1 + 0.5);
+      xend = x1 + grad * (yend - y1);
+
+      ygap = (1 - modf (y1 + 0.5, &db));
+      ix1 = (int) xend;
+      iy1 = (int) yend;
+
+      brightness1 = (1 - modf (xend, &db)) * ygap;
+      brightness2 = modf (xend, &db) * ygap;
+
+      c1 = (unsigned char) (brightness1 * b);
+      c2 = (unsigned char) (brightness2 * b);
+
+      set_col_pixel (ix1, iy1, c1, rr, gg, bb);
+      set_col_pixel (ix1 + 1, iy1, c2, rr, gg, bb);
+
+      xf = xend + grad;
+
+      yend = (int) (y2 + .5);
+      xend = x2 + grad * (yend - y2);
+
+      ygap = 1 - modf (y2 - .5, &db);
+
+      ix2 = (int) xend;
+      iy2 = (int) yend;
+
+      brightness1 = (1 - modf (xend, &db)) * ygap;
+      brightness2 = modf (xend, &db) * ygap;
+
+      c1 = (unsigned char) (brightness1 * b);
+      c2 = (unsigned char) (brightness2 * b);
+
+      set_col_pixel (ix2, iy2, c1, rr, gg, bb);
+      set_col_pixel (ix2 + 1, iy2, c2, rr, gg, bb);
+      for (i = iy1 + 1; i < iy2; i++)
+        {
+          brightness1 = (1 - modf (xf, &db));
+          brightness2 = modf (xf, &db);
+
+          c1 = (unsigned char) (brightness1 * b);
+          c2 = (unsigned char) (brightness2 * b);
+
+          set_col_pixel ((int) xf, i, c1, rr, gg, bb);
+          set_col_pixel ((int) (xf + 1), i, c2, rr, gg, bb);
+
+          xf += grad;
+        }
+    }
+}
+
+/* bresenham line algorithm */
+static void 
+lineBresenham(gint x0, gint y0, gint x1, gint y1, guchar b,
+         guchar rr, guchar gg, guchar bb)
+{
+    int dy = y1 - y0;
+    int dx = x1 - x0;
+    int stepx, stepy;
+
+    if (dy < 0) { dy = -dy;  stepy = -1; } else { stepy = 1; }
+    if (dx < 0) { dx = -dx;  stepx = -1; } else { stepx = 1; }
+    dy <<= 1;                                                  // dy is now 2*dy
+    dx <<= 1;                                                  // dx is now 2*dx
+
+    set_col_pixel ((int) x0, y0, b, rr, gg, bb);
+    if (dx > dy) {
+        int fraction = dy - (dx >> 1);                         // same as 2*dy - dx
+        while (x0 != x1) {
+            if (fraction >= 0) {
+                y0 += stepy;
+                fraction -= dx;                                // same as fraction -= 2*dx
+            }
+            x0 += stepx;
+            fraction += dy;                                    // same as fraction -= 2*dy
+            set_col_pixel ((int) x0, y0, b, rr, gg, bb);
+        }
+    } else {
+        int fraction = dx - (dy >> 1);
+        while (y0 != y1) {
+            if (fraction >= 0) {
+                x0 += stepx;
+                fraction -= dy;
+            }
+            y0 += stepy;
+            fraction += dx;
+            set_col_pixel ((int) x0, y0, b, rr, gg, bb);
+        }
+    }
+}
+
 static void
 change_dial_color(void)
 {
@@ -316,7 +512,7 @@ change_dial_color(void)
 }
 
 static void
-draw_clock(void)
+draw_aclock(void)
 {
    gint tempX, tempY;
    gint h, m, s;
@@ -395,6 +591,100 @@ draw_clock(void)
    aa_line (half_w, 20, tempX, tempY, (unsigned char) 255, c_red[s_color], c_green[s_color], c_blue[s_color]);
 
 }
+
+
+static void
+draw_xclock(void)
+{
+   gint tempX, tempY;
+   gint h, m, s;
+   gint half_w, i;
+
+
+   gdouble        minutes = 20, hours = 12, seconds = 0;
+
+   tm = gkrellm_get_current_time();
+
+   h = tm->tm_hour;
+   m = tm->tm_min;
+   s = tm->tm_sec;
+   hours = (double)h + ((double)m / 60) + ((double)s / 3600);
+   minutes = (double)m + ((double)s / 60);
+   seconds = (double)s;
+
+   if ( cycle && prev_h != -1 && prev_h != h )
+   {
+     change_dial_color();
+     if ( d_color == s_color ) { change_dial_color(); }
+   }
+
+   prev_h = h;
+
+   blank_buf();
+
+   half_w = chart_w / 2;
+
+   if ( half_w > CHART_H ) { half_w = CHART_H; }
+
+
+   lineBresenham(half_w, 0, half_w, 4, 255, c_red[d_color], c_green[d_color], c_blue[d_color]);
+
+   set_col_pixel (half_w + 9, 4, 255, c_red[d_color], c_green[d_color], c_blue[d_color]);
+   set_col_pixel (half_w + 15, 10, 255, c_red[d_color], c_green[d_color], c_blue[d_color]);
+   set_col_pixel (half_w + 15, 28, 255, c_red[d_color], c_green[d_color], c_blue[d_color]);
+   set_col_pixel (half_w + 9, 34, 255, c_red[d_color], c_green[d_color], c_blue[d_color]);
+
+   lineBresenham(half_w, 39, half_w, 35, 255, c_red[d_color], c_green[d_color], c_blue[d_color]);
+
+   set_col_pixel (half_w - 9, 4, 255, c_red[d_color], c_green[d_color], c_blue[d_color]);
+   set_col_pixel (half_w - 15, 10, 255, c_red[d_color], c_green[d_color], c_blue[d_color]); 
+   set_col_pixel (half_w - 15, 28, 255, c_red[d_color], c_green[d_color], c_blue[d_color]);
+   set_col_pixel (half_w - 9, 34, 255, c_red[d_color], c_green[d_color], c_blue[d_color]);
+
+   lineBresenham(half_w - 20, 19, half_w - 16, 19, 255, c_red[d_color], c_green[d_color], c_blue[d_color]);
+
+   lineBresenham(half_w + 20, 19, half_w + 16, 19, 255, c_red[d_color], c_green[d_color], c_blue[d_color]);
+   
+   
+   tempX =  (int)(sin((hours * 3.14159 * 2) / 12) * 12 );
+   tempY = -(int)(cos((hours * 3.14159 * 2) / 12) * 12 );
+   Wu_line(half_w - tempY / 4, 19 + tempX / 4, half_w + tempX, 19 + tempY, (unsigned char) 255, c_red[d_color], c_green[d_color], c_blue[d_color]);
+   Wu_line(half_w + tempY / 4, 19 - tempX / 4, half_w + tempX, 19 + tempY, (unsigned char) 255, c_red[d_color], c_green[d_color], c_blue[d_color]);
+   Wu_line(half_w - tempY / 4, 19 + tempX / 4, half_w - tempX / 4, 19 - tempY / 4, (unsigned char) 255, c_red[d_color], c_green[d_color], c_blue[d_color]);
+   Wu_line(half_w + tempY / 4, 19 - tempX / 4, half_w - tempX / 4, 19 - tempY / 4, (unsigned char) 255, c_red[d_color], c_green[d_color], c_blue[d_color]);
+   
+   tempX =  (int)(sin((minutes * 3.14159 * 2) / 60) * 18 );
+   tempY = -(int)(cos((minutes * 3.14159 * 2) / 60) * 18 );
+   Wu_line(half_w - tempY / 9, 19 + tempX / 9, half_w + tempX, 19 + tempY, (unsigned char) 255, c_red[d_color], c_green[d_color], c_blue[d_color]);
+   Wu_line(half_w + tempY / 9, 19 - tempX / 9, half_w + tempX, 19 + tempY, (unsigned char) 255, c_red[d_color], c_green[d_color], c_blue[d_color]);
+
+/* normal seconds hand
+   tempX =  (int)(sin((seconds * 3.14159 * 2) / 60) * 19 );
+   tempY = -(int)(cos((seconds * 3.14159 * 2) / 60) * 19 );
+   Wu_line(half_w, 19, half_w + tempX, 19 + tempY, (unsigned char) 255, c_red[s_color], c_green[s_color], c_blue[s_color]);
+*/
+
+/* BBC Clock style seconds - not so legible at this size */
+   tempX =  (int)(sin((seconds * 3.14159 * 2) / 60) * 19 );
+   tempY = -(int)(cos((seconds * 3.14159 * 2) / 60) * 19 );
+   Wu_line(half_w + tempX / 2 , 19  + tempY / 2, half_w + tempX, 19 + tempY, (unsigned char) 255, c_red[s_color], c_green[s_color], c_blue[s_color]);
+
+   for (i = 0; i < seconds + 1; i++)
+      {
+       tempX =  (int)(sin((i * 3.14159 * 2) / 60) * 19 );
+       tempY = -(int)(cos((i * 3.14159 * 2) / 60) * 19 );
+       set_col_pixel (half_w + tempX, 19 + tempY, 255, c_red[s_color], c_green[s_color], c_blue[s_color]);
+      }
+
+}
+
+static void
+draw_clock(void)
+{
+   if ( clock_type ) { draw_xclock(); }
+   else { draw_aclock(); }
+}
+
 static gint
 chart_expose_event(GtkWidget *widget, GdkEventExpose *ev)
 {
@@ -506,6 +796,8 @@ save_aclock_config (FILE *f)
             d_color);
     fprintf(f, "%s s_color %d\n", PLUGIN_CONFIG_KEYWORD,
             s_color);
+    fprintf(f, "%s clock_type %d\n", PLUGIN_CONFIG_KEYWORD,
+            clock_type);
 }
 
 static void
@@ -523,6 +815,8 @@ load_aclock_config (gchar *arg)
             sscanf(item, "%d\n", &(d_color));
         if (strcmp(config, "s_color") == 0)
             sscanf(item, "%d\n", &(s_color));
+        if (strcmp(config, "clock_type") == 0)
+            sscanf(item, "%d\n", &(clock_type));
     }
 
 }
@@ -563,11 +857,20 @@ static void cycle_clicked(GtkButton* button )
     cycle = GTK_TOGGLE_BUTTON(cycle_option)->active;
 }
 
+static void
+cb_clock_type(GtkWidget *button, gpointer data)
+{
+        gint    i = GPOINTER_TO_INT(data);
+
+        if (GTK_TOGGLE_BUTTON(button)->active)
+                clock_type = i;
+}
+
 static void
 create_aclock_tab (GtkWidget *tab)
 {
-    GtkWidget *laptop, *about_label, *label, *frame, *hbox, *vbox;
-	GtkWidget *s_label, *text, *info_window ;
+    GtkWidget *laptop, *about_label, *label, *frame, *hbox, *vbox, *ybox;
+	GtkWidget *s_label, *text, *info_window, *button ;
 
 	GList *d_items = NULL;
 	GList *s_items = NULL;
@@ -583,6 +886,7 @@ create_aclock_tab (GtkWidget *tab)
       "and right-click opens configuration windws.\n\n",
       "<b> Options \n\n" ,
       "Cycle Dial Color - when checked will change dial color every hour.\n\n",
+      "Clock Type - choose from Aclock and Xclock type .\n\n",
       "Select Dial Color  from drop down .\n",
       "Select Seconds Color from drop down .\n",
     };
@@ -599,9 +903,9 @@ create_aclock_tab (GtkWidget *tab)
     vbox = gtk_vbox_new( FALSE, 0 );
     gtk_container_border_width(GTK_CONTAINER(vbox), 3);
 
-	/* Dial Color */
 
-    hbox = gtk_hbox_new(FALSE, 0);
+	/* Cycle Dial Color */
+
 
     cycle_option = gtk_check_button_new_with_label( "Cycle Dial Color" );
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cycle_option),
@@ -612,7 +916,31 @@ create_aclock_tab (GtkWidget *tab)
 
     gtk_box_pack_start(GTK_BOX(vbox), cycle_option, FALSE, FALSE, 0);
 
+    /* Clock Type */
+    
+     ybox = gkrellm_gtk_framed_vbox(vbox, _("Clock Type"),
+                        4, FALSE, 0, 2);
+        
+     hbox = gtk_hbox_new(FALSE, 0);
+     gtk_box_pack_start(GTK_BOX(ybox), hbox, FALSE, FALSE, 0);
+    
+     clock_type_option[0] = gtk_radio_button_new_with_label(NULL, _("Aclock"));
+     gtk_box_pack_start(GTK_BOX(hbox), clock_type_option[0], TRUE, TRUE, 0);
+     clock_type_option[1] = gtk_radio_button_new_with_label_from_widget(
+                                        GTK_RADIO_BUTTON(clock_type_option[0]), _("Xclock"));
+     gtk_box_pack_start(GTK_BOX(hbox), clock_type_option[1], TRUE, TRUE, 0);
+    
+     button = clock_type_option[clock_type];
+     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
+        
+     for (i = 0; i < 2; ++i) {
+                g_signal_connect(G_OBJECT(clock_type_option[i]), "toggled",
+                                        G_CALLBACK(cb_clock_type), GINT_TO_POINTER(i));
+     }      
 
+
+	/* Dial Color */
+    hbox = gtk_hbox_new(FALSE, 0);
     label = gtk_label_new("Dial Color " );
 
     for ( i = 0; i < MAX_COLORS; i++ )
@@ -679,7 +1007,7 @@ create_aclock_tab (GtkWidget *tab)
         "GKrellAclock %s\n" \
         "GKrellM Aclock Plugin\n" \
         "\n" \
-        "Copyright (C) 2002 M.R.Muthu Kumar\n" \
+        "Copyright (C) 2005 M.R.Muthu Kumar\n" \
         "m_muthukumar@users.sourceforge.net\n" \
         "\n" \
         "Released under the GNU Public License\n" \
@@ -729,7 +1057,6 @@ gkrellm_init_plugin()
       gint i;
 #if defined(WIN32)
     callbacks = calls;
-    pwin32GK = calls->GK;
 #endif
 
 	/* If this call is made, the background and krell images for this plugin
@@ -748,6 +1075,8 @@ gkrellm_init_plugin()
     d_color = WHITE;
 	s_color = RED;
 	cycle   = 1;
+        clock_type = X_CLOCK;
+
 
 	chart_w = gkrellm_chart_width();
 
